@@ -13,6 +13,7 @@ module System.Directory.OsPath.Streaming
   ( DirStream
   , openDirStream
   , readDirStream
+  , readDirStreamFull
   , closeDirStream
 
   -- * File types
@@ -23,6 +24,7 @@ module System.Directory.OsPath.Streaming
 
   -- * Get directory contents
   , getDirectoryContentsRecursive
+  , getDirectoryContentsWithFilterRecursive
 
   , listContentsRecFold
 
