@@ -14,10 +14,12 @@
 module TestMain (main) where
 
 import Control.Exception
+import Data.Functor.Identity (Identity(..))
 import qualified Data.List as L
 import System.OsPath
 
 import System.Directory.OsPath.Streaming
+import System.Directory.OsPath.Types
 
 import Test.Tasty
 import Test.Tasty.HUnit
@@ -45,6 +47,18 @@ tests = testGroup "Tests"
         return $ L.sort [w, x, y, z]
       res @?= [([osp|bar.txt|], File Regular), ([osp|baz.txt|], File Regular), ([osp|bin|], Directory Regular), ([osp|foo.txt|], File Regular)]
 
+  , testCase "readDirStreamFull" $ do
+      let dir = [osp|test/filesystem|]
+      res <- bracket (openDirStream dir) closeDirStream $ \ds -> do
+        Just w <- readDirStreamFull ds
+        Just x <- readDirStreamFull ds
+        Just y <- readDirStreamFull ds
+        Just z <- readDirStreamFull ds
+        return $ L.sort [w, x, y, z]
+      res @?= map
+        (\(x, y) -> (dir </> x, Basename x, y))
+        [([osp|bar.txt|], File Regular), ([osp|baz.txt|], File Regular), ([osp|bin|], Directory Regular), ([osp|foo.txt|], File Regular)]
+
   , testGroup "getFileType general"
       [ testCase "file" $ do
           getFileType [osp|directory-ospath-streaming.cabal|] >>= (@?= File Regular)
@@ -52,9 +66,39 @@ tests = testGroup "Tests"
           getFileType [osp|test|] >>= (@?= Directory Regular)
       ]
 
-  , testCase "getDirectoryContentsRecursive" $ do
-    res <- L.sort <$> getDirectoryContentsRecursive [osp|test/filesystem|]
-    res @?= [([osp|bar.txt|], File Regular), ([osp|baz.txt|], File Regular), ([osp|bin|], Directory Regular), ([osp|bin|] </> [osp|bin.txt|], File Regular), ([osp|foo.txt|], File Regular)]
+  , testGroup "contents"
+      [ testCase "getDirectoryContentsRecursive" $ do
+          res <- L.sort <$> getDirectoryContentsRecursive [osp|test/filesystem|]
+          res @?= [([osp|bar.txt|], File Regular), ([osp|baz.txt|], File Regular), ([osp|bin|], Directory Regular), ([osp|bin|] </> [osp|bin.txt|], File Regular), ([osp|foo.txt|], File Regular)]
+
+      , testCase "getDirectoryContentsWithFilterRecursive 1" $ do
+          res <- L.sort <$> getDirectoryContentsWithFilterRecursive (\_ _ -> True) (const True) [osp|test/filesystem|]
+          res @?= [([osp|bar.txt|], File Regular), ([osp|baz.txt|], File Regular), ([osp|bin|], Directory Regular), ([osp|bin|] </> [osp|bin.txt|], File Regular), ([osp|foo.txt|], File Regular)]
+
+      , testCase "getDirectoryContentsWithFilterRecursive 2" $ do
+          res <- L.sort <$> getDirectoryContentsWithFilterRecursive (\x _ -> x /= Basename [osp|bin|]) (const True) [osp|test/filesystem|]
+          res @?= [([osp|bar.txt|], File Regular), ([osp|baz.txt|], File Regular), ([osp|bin|], Directory Regular), ([osp|foo.txt|], File Regular)]
+
+      , testCase "getDirectoryContentsWithFilterRecursive 3" $ do
+          res <- L.sort <$> getDirectoryContentsWithFilterRecursive (\_ _ -> True) (`elem` [Basename [osp|foo.txt|], Basename [osp|bin|], Basename [osp|bin.txt|]]) [osp|test/filesystem|]
+          res @?= [([osp|bin|], Directory Regular), ([osp|bin|] </> [osp|bin.txt|], File Regular), ([osp|foo.txt|], File Regular)]
+
+      , testCase "listContentsRecFold" $ do
+          let dir = [osp|test/filesystem|]
+          res <- fmap L.sort $
+            listContentsRecFold
+              Nothing
+              (\a b c d e cons rec rest -> cons (a, b, c, d, Directory e) $ rec rest)
+              (\a b c d e -> pure $ Just (a, b, c, d, e))
+              (Identity dir)
+          res @?=
+            [ (dir </> [osp|bar.txt|],                dir, Relative [osp|bar.txt|],                  Basename [osp|bar.txt|], File Regular)
+            , (dir </> [osp|baz.txt|],                dir, Relative [osp|baz.txt|],                  Basename [osp|baz.txt|], File Regular)
+            , (dir </> [osp|bin|],                    dir, Relative [osp|bin|],                      Basename [osp|bin|],     Directory Regular)
+            , (dir </> [osp|bin|] </> [osp|bin.txt|], dir, Relative $ [osp|bin|] </> [osp|bin.txt|], Basename [osp|bin.txt|], File Regular)
+            , (dir </> [osp|foo.txt|],                dir, Relative [osp|foo.txt|],                  Basename [osp|foo.txt|], File Regular)
+            ]
+      ]
 
 #ifndef mingw32_HOST_OS
   , withResource
@@ -62,7 +106,7 @@ tests = testGroup "Tests"
         tmp <- getTemporaryDirectory >>= canonicalizePath
         createFreshTempDir tmp [osp|test|])
       removeDirectoryRecursive
-      $ \mkTmpDir -> testGroup "getFileType unix"
+    $ \mkTmpDir -> testGroup "getFileType unix"
         [ testCase "file symlink" $ do
             tmp     <- mkTmpDir
             currDir <- getCurrentDirectory
